@@ -17,13 +17,13 @@ import logging
 from neutronclient.common import utils as n_utils
 from neutronclient.i18n import _
 from neutronclient.neutron import v2_0 as neutronV20
+from neutronclient.v2_0 import client as neutron_client
 from oslo_serialization import jsonutils
 
 from gbpclient.common import utils
 
-
-neutronV20.UUID_PATTERN = neutronV20.UUID_PATTERN + (
-    '|auto' + neutronV20.HEX_ELEM + '{32}')
+neutron_client.UUID_PATTERN = neutron_client.UUID_PATTERN + (
+    '|auto' + neutron_client.HEX_ELEM + '{32}')
 
 
 def _format_fixed_ips(pt):
@@ -179,7 +179,8 @@ class ListPolicyTargetGroup(neutronV20.ListCommand):
 
     resource = 'policy_target_group'
     log = logging.getLogger(__name__ + '.ListPolicyTargetGroup')
-    list_columns = ['id', 'name', 'description', 'l2_policy_id', 'subnets']
+    list_columns = ['id', 'name', 'description', 'application_policy_group_id',
+                    'l2_policy_id', 'subnets']
     pagination_support = True
     sorting_support = True
 
@@ -205,6 +206,11 @@ class CreatePolicyTargetGroup(neutronV20.CreateCommand):
             'name', metavar='NAME',
             help=_('Name of Policy Target Group to create '
                    '(required argument)'))
+        parser.add_argument(
+            '--application-policy-group', metavar='APPLICATION_POLICY_GROUP',
+            default='',
+            help=_('Application Policy Group UUID (optional, default '
+                   'is None)'))
         parser.add_argument(
             '--l2-policy', metavar='L2_POLICY',
             default='',
@@ -241,6 +247,12 @@ class CreatePolicyTargetGroup(neutronV20.CreateCommand):
                     self.get_client(), 'l2_policy',
                     parsed_args.l2_policy)
 
+        if parsed_args.application_policy_group:
+            body[self.resource]['application_policy_group_id'] = (
+                neutronV20.find_resourceid_by_name_or_id(
+                    self.get_client(), 'application_policy_group',
+                    parsed_args.application_policy_group))
+
         if parsed_args.network_service_policy:
             body[self.resource]['network_service_policy_id'] = \
                 neutronV20.find_resourceid_by_name_or_id(
@@ -292,6 +304,9 @@ class UpdatePolicyTargetGroup(neutronV20.UpdateCommand):
         parser.add_argument(
             '--l2-policy', metavar='L2_POLICY',
             help=_('New L2 policy'))
+        parser.add_argument(
+            '--application-policy-group', metavar='APPLICATION_POLICY_GROUP',
+            help=_('New Application Policy Group'))
         parser.add_argument(
             '--network-service-policy', metavar='NETWORK_SERVICE_POLICY',
             help=_('New Network Service Policy'))
@@ -326,6 +341,14 @@ class UpdatePolicyTargetGroup(neutronV20.UpdateCommand):
                     self.get_client(), 'l2_policy',
                     parsed_args.l2_policy)
 
+        if parsed_args.application_policy_group == '':
+            body[self.resource]['application_policy_group_id'] = None
+        elif parsed_args.application_policy_group:
+            body[self.resource]['application_policy_group_id'] = (
+                neutronV20.find_resourceid_by_name_or_id(
+                    self.get_client(), 'application_policy_group',
+                    parsed_args.application_policy_group))
+
         if parsed_args.network_service_policy == '':
             body[self.resource]['network_service_policy_id'] = None
         elif parsed_args.network_service_policy:
@@ -674,6 +697,85 @@ class UpdateL3Policy(neutronV20.UpdateCommand):
         return body
 
 
+class ListApplicationPolicyGroup(neutronV20.ListCommand):
+    """List l3_policies that belong to a given tenant."""
+
+    resource = 'application_policy_group'
+    log = logging.getLogger(__name__ + '.ListApplicationPolicyGroup')
+    _formatters = {}
+    list_columns = ['id', 'name', 'description', 'shared']
+    pagination_support = True
+    sorting_support = True
+
+
+class ShowApplicationPolicyGroup(neutronV20.ShowCommand):
+    """Show information of a given Application Policy Group."""
+
+    resource = 'application_policy_group'
+    log = logging.getLogger(__name__ + '.ShowApplicationPolicyGroup')
+
+
+class CreateApplicationPolicyGroup(neutronV20.CreateCommand):
+    """Create a Application Policy Group for a given tenant."""
+
+    resource = 'application_policy_group'
+    log = logging.getLogger(__name__ + '.CreateApplicationPolicyGroup')
+
+    def add_known_arguments(self, parser):
+        parser.add_argument(
+            '--description',
+            help=_('Description of the Application Policy Group'))
+        parser.add_argument(
+            'name', metavar='NAME',
+            help=_('Name of L3 policy to create (required argument)'))
+        n_utils.add_boolean_argument(
+            parser, '--shared', dest='shared',
+            help=_('Enable or disable resource sharing, default is False'))
+
+    def args2body(self, parsed_args):
+        body = {self.resource: {}, }
+
+        neutronV20.update_dict(parsed_args, body[self.resource],
+                               ['name', 'tenant_id', 'description',
+                                'shared'])
+
+        return body
+
+
+class DeleteApplicationPolicyGroup(neutronV20.DeleteCommand):
+    """Delete a given Application Policy Group."""
+
+    resource = 'application_policy_group'
+    log = logging.getLogger(__name__ + '.DeleteApplicationPolicyGroup')
+
+
+class UpdateApplicationPolicyGroup(neutronV20.UpdateCommand):
+    """Update Application Policy Group's information."""
+
+    resource = 'application_policy_group'
+    log = logging.getLogger(__name__ + '.UpdateApplicationPolicyGroup')
+
+    def add_known_arguments(self, parser):
+        parser.add_argument(
+            '--description',
+            help=_('New description of the Application Policy Group'))
+        parser.add_argument(
+            '--name',
+            help=_('New name of the Application Policy Group'))
+        n_utils.add_boolean_argument(
+            parser, '--shared', dest='shared',
+            help=_('Enable or disable resource sharing'))
+
+    def args2body(self, parsed_args):
+        body = {self.resource: {}, }
+
+        neutronV20.update_dict(parsed_args, body[self.resource],
+                               ['name', 'tenant_id', 'description',
+                                'shared'])
+
+        return body
+
+
 class ListNetworkServicePolicy(neutronV20.ListCommand):
     """List Network Service Policies that belong to a given tenant."""
 
